function [iaf,varides,niv,fro]=apeles(B,Bm,met,trayest)

% APELES carga las funciones de cuantizado no unifrome 
% (funciones de distribucion en el caso estadistico) y
% disea el cuantizador con 2^B niveles por coeficiente
%
% Atencion!. El programa supone que en el fichero de datos
% hay tres variables: 'FF' 'dom' y 'vari'.
%
% Se supone que 'FF' contiene (en cada fila) las funciones de 
% cuantizado no uniforme, 'dom' contiene los dominios de estas
% funciones, y 'vari' contiene la funcion que da el numero 
% relativo de bits que hay que emplear en cada coeficiente 
% (ver APELO).
%
% Podemos distribuir la informacion por coeficiente de dos formas
%  
%  metodo 1: aplicando un factor a la CIAF hasta obtener el 
%            valor en bits/pix deseado.
%
%  metodo 2: aplicando la expresion del Gersho (y el Tekalp y el
%            Watanave...) para la varianza iterando hasta que la    
%            parte positiva de lo que tiene que dar.
%
%
%              dB     B     1                 1
%             ---- = --- + --- Log2( var ) - ---- sum( log2(var) )
%              df     N     2                 2N
%
%  metodo 3: supone que tenemos varianza y csf por separado y aplica iterativamente
%            la expresion del Gersho a varianza*csf
%
%              dB     B     1                     1
%             ---- = --- + --- Log2( var*csf ) - ---- sum( log2(var*csf) )
%              df     N     2                     2N
%
%  metodo 4: supone que tenemos varianza y csf por separado y aplica iterativamente
%            la expresion del Gersho a la varianza y luego multiplica el numero 
%            de BITS/COEFF por la csf
%
%                    /                                              \
%              dB    |  B     1                 1                   | 
%             ---- = | --- + --- Log2( var ) - ---- sum( log2(var) )| * csf 
%              df    |  N     2                 2N                  |
%                    \                                              /
%
%  metodo 5: supone que tenemos varianza y csf por separado y aplica iterativamente
%            la expresion del Gersho a la varianza y luego pesa el numero de
%            NIV/COEFF por la csf
%
%                            /                                              \
%              dN            |  B     1                 1                   | 
%             ---- = csf * 2^| --- + --- Log2( var ) - ---- sum( log2(var) )| 
%              df            |  N     2                 2N                  |
%                            \                                              /
%
%  metodo 6: distribucion de numero de NIV/COEFF por la csf
%            
%
%                          dN       
%                         ---- = csf
%                          df       
%                       
%  metodo 7: Algoritmo voraz de Gersho/Ramstad que cumple la no negatividad y la 
%            distribucion entera de niveles.
%
%            Si tocan B bits, N=2^B niveles, para codificar un bloque de coeficientes,
%            repartelos uno tras otro, en funcion de que cuantizador sea el que genere
%            mas DISTORSION CUADRATICA para el numero de niveles con el que cuenta en
%            ese momento.
%            Sabemos que la DISTORSION CUADRATICA de cada coeficiente al que se asignan
%            N niveles optimamente repartidos es:
%
%                              1
%                     Df(N)= ----- varf . Hf
%                             N^2
%
%            donde varf es la varianza del coeficiente y Hf es la integral:
%
%                             _      
%                       /    /                              \ 3
%                   1   |    |                              | 
%             Hf = ---- |    |   (Wf(sigm*a)*f(a))^(1/3) da | 
%                   12  |    |                              |
%                       \   _/                              /
%
%            
%            varf y Hf estan almacenadas en el fichero de datos (vease MAL2BIEN)
%            como vari=log(varf*Hf)/log(2).
%
%  metodo 8: Algoritmo voraz de Gersho/Ramstad que cumple la no negatividad y la 
%            distribucion entera de niveles.
%
%            Si tocan B bits, N=2^B niveles, para codificar un bloque de coeficientes,
%            repartelos uno tras otro, en funcion de que cuantizador sea el que genere
%            mas DISTORSION NO CUADRATICA para el numero de niveles con el que cuenta en
%            ese momento.
%            Sabemos que la DISTORSION NO CUADRATICA de cada coeficiente al que se asignan
%            N niveles optimamente repartidos es:
%
%                              1
%                     Df(N)= ----- sigmf . Hf
%                              N
%
%            donde sigmf es la desviacion standard del coeficiente y Hf es la integral:
%
%                             _      
%                       /    /                                    \ 2
%                   1   |    |                                    | 
%             Hf = ---  |    |   (Wf(sigm*a)^(1/2)*f(a))^(1/2) da | 
%                   4   |    |                                    |
%                       \   _/                                    /
%
%            
%            sigmf y Hf estan almacenadas en el fichero de datos (vease MAL2BIE2)
%            como vari=log(sigmf*Hf)/log(2).
% 
% USO: [iaf,bit_por_frec,niv,fro]=apeles(B,Bmax,metodo_bit_alloc,'fichero datos');
% 

eval(['load ',trayest,';'])

s=size(dom);
s=s(1);

s=ceil(1+sqrt((s-1)^2/2));

vari=[0 vari(2:length(vari))];

vari=simrev(vari);
Bobj=B;
if met==1
   vari=vari/(sum(sum(vari(1:s,1:s))));
   varides=B*vari*s*s;
   clear vari
   varides=varides(1,:);
   cond=(varides<=Bm);
   anticond=cond<1;
   nniv=cond.*2.^(varides)+anticond*2.^Bm;
   [iaf,ciaf,fff,FFFF,contr]=iafest2(trayest,Bobj,met,0);
elseif met==2
   c=10;
   while abs(c)>0.01
         bari=B+0.5*vari-(0.5/s^2)*sum(sum(vari(1:s,1:s)));
         bari=bari.*(bari>0);
         c=Bobj-sum(sum(bari(1:s,1:s)))/s^2;
         B=B+c;
         varides=bari;
   end
   varides=varides(1,:);
   cond=(varides<=Bm);
   anticond=cond<1;
   nniv=cond.*2.^(varides)+anticond*2.^Bm;
   [iaf,ciaf,fff,FFFF,contr]=iafest2(trayest,Bobj,met,0);
elseif met==3
   csf=[0 csf(2:length(csf))];
   csf=simrev(csf);
   csf=csf/maxi(csf); 
   vari=vari.*csf;
   varii=vari(1,:);
   v=find(varii>0);
   varii(v)=log(varii(v))/log(2);
   vari=simrev(varii);
   c=10;
   while abs(c)>0.01
         bari=B+0.5*vari-(0.5/s^2)*sum(sum(vari(1:s,1:s)));
         bari=bari.*(bari>0);
         c=Bobj-sum(sum(bari(1:s,1:s)))/s^2;
         B=B+c;
         varides=bari;
   end
   varides=varides(1,:);
   cond=(varides<=Bm);
   anticond=cond<1;
   nniv=cond.*2.^(varides)+anticond*2.^Bm;
   [iaf,ciaf,fff,FFFF,contr]=iafest2(trayest,Bobj,met,0);
elseif met==4
   csf=[0 csf(2:length(csf))];
   csf=simrev(csf);
   csf=(s^2)*csf/sum(sum(csf)); 
   v=find(vari>0);
   vari(v)=log(vari(v))/log(2);
   vari=simrev(vari);
   c=10;
   while abs(c)>0.01
         bari=B+0.5*vari-(0.5/s^2)*sum(sum(vari(1:s,1:s)));
         bari=bari.*(bari>0).*csf;
         c=Bobj-sum(sum(bari(1:s,1:s)))/s^2;
         B=B+c;
         varides=bari;
   end
   varides=varides(1,:);
   cond=(varides<=Bm);
   anticond=cond<1;
   nniv=cond.*2.^(varides)+anticond*2.^Bm;
   [iaf,ciaf,fff,FFFF,contr]=iafest2(trayest,Bobj,met,0);
elseif met==5
   csf=[0 csf(2:length(csf))];
   csf=simrev(csf);
   csf=(s^2)*csf/sum(sum(csf));
   v=find(vari>0);
   vari(v)=log(vari(v))/log(2);
   vari=simrev(vari);
   c=10;
   while abs(c)>0.01
         bari=B+0.5*vari-(0.5/s^2)*sum(sum(vari(1:s,1:s)));
         bari=bari.*(bari>0);
         bari=(2.^(bari)).*csf;
         bari=bari(1,:);
         v=find(bari>0); 
         bari(v)=log(bari(v))/log(2);
         bari=bari.*(bari>0);
         bari=simrev(bari);
         c=Bobj-sum(sum(bari(1:s,1:s)))/s^2;
         B=B+c;
         varides=bari;
   end
   varides=varides(1,:);
   cond=(varides<=Bm);
   anticond=cond<1;
   nniv=cond.*(2.^varides)+anticond*(2.^Bm);
   [iaf,ciaf,fff,FFFF,contr]=iafest2(trayest,Bobj,met,0);
elseif met==6
   csf=[0 csf(2:length(csf))];
   csf=simrev(csf);
   csf=csf/(sum(sum(csf(1:s,1:s))));
   varides=(2^B)*csf*s*s;
%  log(sum(sum(floor(varides(1:s,1:s))))/s^2)/log(2)
%  pause
   varides=varides(1,:);
   cond=(varides<=(2^Bm));
   anticond=cond<1;
   nniv=cond.*varides+anticond*(2.^Bm);
   [iaf,ciaf,fff,FFFF,contr]=iafest2(trayest,Bobj,met,0);
else
   N=ceil(s*s*2^(B));
   N-s*s
   varH=2.^(vari);   
%   varH=varH(1:s,1:s);
   varH=zigzag(varH);
   nniv2d=ones(length(varH),1);
   if met==7  
      for i=1:N-(s*s),
          m=max(varH./(nniv2d.^2));
          indic=find(varH./(nniv2d.^2)==m);
          I=indic(round(1+rand*(length(indic)-1)));
          nniv2d(I)=nniv2d(I)+1;
      end 
   else 
      for i=1:N-(s*s),
          m=max(varH./nniv2d);
          indic=find((varH./nniv2d)==m);
          I=indic(round(1+rand*(length(indic)-1)));
          nniv2d(I)=nniv2d(I)+1;
      end 
   end
   nniv2d=dezigzag(nniv2d);
   nniv=round(0.5*(nniv2d(1,:)+nniv2d(:,1)'));
   varides=log(nniv)/log(2);

   %%%%% ROLLO PARA DEFINIR LA IAF A 'EFECTOS GRAFICOS': 
   %
   %     Consiste en:
   %
   %       - Definir todas las funciones densidad de niveles y funciones companding
   %         en el mismo dominio
   %
   %       - Multiplicar cada funcion densidad (ya normalizada y en un dominio comun)
   %         por el numero de niveles por coeficiente que le toquen
   %

   ncontr=500;          % Aqui se podria definir un cierto numero de amplitudes donde computar 
                        % las funciones densidad de probabilidad para cada coeficiente...
                        % Por defecto se toma el promedio de numero de puntos en el que estan 
                        % definidas.

   s=size(dom);
   dm=dom(s(1),s(2));
   ddd=dom.*(dom<dm);
   dd=min(s(2)-sum((dom==dm)'));
   cmax=maxi(ddd(2:s(1),:));
   if ncontr==0
      ncontr=floor((dd+s(2))/2);
   end
   contr=linspace(0,cmax,ncontr);
   FFFF=zeros(1,ncontr);

   for i=2:s(1)
       domi=dom(i,:);
       domi=domi(find(domi<dm));
       l=length(domi);
       FFF=FF(i,:);
       FFF=FFF(1:l);
       a=ncontr-l;
       if a>=1
          if max(domi)<cmax
             d1=linspace(max(domi),cmax,a+1);
             domi=[domi d1(2:length(d1))];
             FFF=[FFF ones(1,a)];
             FFF=(interp1(domi,FFF,contr))';
          else
             FFF=(interp1(domi,FFF,contr))';
          end
       else
          if max(domi)<cmax
             domi=[domi cmax];
             FFF=[FFF 1];
             FFF=(interp1(domi,FFF,contr))';
          else
             FFF=(interp1(domi,FFF,contr))';
          end  
       end
       FFFF(i,:)=FFF;
   end
   fff=-(FFFF(:,1:ncontr-1)-FFFF(:,2:ncontr));
   fff=[fff zeros(s(1),1)];
   Norm=sum(fff');
   Norm(1)=1;
   Norm=(Norm')*ones(1,ncontr);
   fff=fff./Norm;
   FFFF=(cumsum(fff'))';

   nnnniv=(nniv')*ones(1,ncontr);
   iaf=nnnniv.*fff;
   ciaf=(sum(iaf'))';

   %%%%%

end

[niv,fro]=apelin(floor(nniv),dom,FF);
%plot(niv,'c.')
