function [Dacum]=buscad1(A,B,p0,v,N,fac)
 
% BUSCAD1 estima el movimiento del punto p[i j] del fotograma S(t-1) (y en su caso 
% de su entorno) mediante Block Matching Algorithm (BMA). El BMA consiste en calcular 
% la correlacion entre unas vecindades V centradas en [i j] para unos ciertos valores 
% del desplazamiento. El vector desplazamiento estimado sera aquel que maximice la co-
% rrelacion. La gracia del metodo es que no se calcula la correlacion para todo D, 
% sino que se realiza una busqueda del D_max en N pasos.
% En el paso i se calcula la correlacion en los puntos Di-1 +(-) 2^(N-i) (donde Di-1
% es el desplazamiento que maximizo la correlacion en el paso anterior) de forma que
% en los primeros pasos se busca en zonas muy lejanas y posteriormante se va refinando
% esa aproximacion grosera. Cuanto mas alto sea el valor de N, empezara buscando por
% desplazamientos mas grandes (lo cual, a veces puede resultar contraproducente). Por
% eso introducimos un factor adicional (fac) para controlar la longitud de los vectores
% prueba.
% Si fac=1 tendremos la busqueda en N pasos standard (Dufaux 95), variando el factor, po-
% demos mantener el numero de iteraciones de la busqueda adaptando la longitud del vector
% prueba a nuestras necesidades.
% Si fac=1 la busqueda da desplazamientos enteros (no se interpola), pero si no es asi, 
% podemos tener vectores prueba de componentes no enteras -> tendremos que interpolar.
%
% USO: D=buscad1(s(t),s(t-1),[i j],dimens_vecindad[m n],N,fac);
 
p=p0;
Dacum=[0 0];
if fac~=1
   inte=1;
else
   inte=0;
end
for i=1:N
      D(1,:)=Dacum+[0 0];
      D(2,:)=Dacum+fac*[0 2^(N-i)];
      D(3,:)=Dacum+fac*[2^(N-i) 2^(N-i)];       
      D(4,:)=Dacum+fac*[2^(N-i) 0];       
      D(5,:)=Dacum+fac*[-2^(N-i) 2^(N-i)];       
      D(6,:)=Dacum+fac*[-2^(N-i) 0];       
      D(7,:)=Dacum+fac*[-2^(N-i) -2^(N-i)];       
      D(8,:)=Dacum+fac*[0 -2^(N-i)];
      D(9,:)=Dacum+fac*[2^(N-i) -2^(N-i)];
%      if i==1
         in=1;
%     else
%        in=2;
%        cc(1)=0; 
%     end
      for j=in:9
         [c,AA,BB]=correpto(A,B,p,v,D(j,:),inte);
         cc(j)=c;
      end
      [m,pos]=max(cc);
      Dacum=D(pos,:);
end
 