function [H,niveles,fronteras]=entroloc(s,pto,lados,Ncuan,frei,calvq,niveles,fronteras,facfrec,ncontr,nolin,estab,B,bitmax,fc,cc,crit,fijarmax,inclcsf,frecbaja);
                      
% ENTROLOC calcula la entropia de una seal 2D en un entorno rectangular de un punto dado.
%               
% El objeto de esta funcion es servir para evaluar la conveniencia del incremento de la pre-
% cision en el calculo del flujo optico para codificacion de video.
% En codificacion de video se trabaja con dos tipos de estructuras de subbloques para analizar
% los frames: 
%           
%    * Una, en el caso de la estimacion del movimiento (que puede ser fija como en el MPEG o 
%      adaptativa como en el caso para el que se utiliza ENTROLOC).
%               
%    * Y otra, en el caso de la cuantizacion que tambien puede ser adaptativa pero que aqui
%      consideraremos fija (como en MPEG). Siendo fija, viene determinada por un solo parame-
%      tro: el nivel de resolucion de la estructura de cuantizacion (Ncuan).
%               
% Para calcular la entropia en un entorno del bloque dividido (o por dividir) consideraremos dos
% opciones:
%            
%    * Una, calcularla en un entorno estricto del bloque problema (no considerando como va a co-
%      dificarse posteriormente la seal de error).
%                 
%    * Y otra, calcularla los subbloques de la estructura de cuantificacion que intersecten con 
%      el entorno del bloque problema (considerando 'en parte' como va a codificarse posterior-
%      mente la seal de error).
%                   
% La seleccion de las opciones se hace atraves del parametro Ncuan:
%                    
%    * Ncuan > 0  ->  Estamos definiendo una determinada estructura para la cuantizacion.
%                     Dicha estructura sera la utilizada en el calculo de entropias.
%                
%    * Ncuan = 0  ->  Estariamos diciendo que se cuantiza todo el frame considerando un solo
%                     bloque. Como esto no suele ser asi (!), utilizamos Ncuan = 0 para selec-
%                     cionar el modo 'calculo de la entropia en un entorno pasando de como se
%                     haga la cuantizacion posterior'.
%               
% ENTROLOC permite calcular:
%             
%    * La 'entropia estandard': expresion de Shannon en el dominio espacial.
%          
%    * La 'entropia perceptual': entropia de la transformada cuantizada mediante la IAF
%           
% Cuando ENTROLOC cuantiza la seal (en el modo frecuencial), puede calcular el cuantizador
% o pasarselo diseado de modo que solo tenga que aplicarlo (con lo que ira mas rapido).
%        
% El modo frecuencial se selecciona introduciendo un 1 en la variable 'frecuencial?'.
% El computo de la entropia perceptual lleva asiciado un VQ que habra que introducir
% o calcular.
% Para elegir calcular el VQ (porque no se tiene uno previamente calculado) introducir 
% un 1 en la variable 'calc_VQ?'. En este caso habra que pasar los parametros necesarios
% para el diseo del VQ (Ver JOTAPIP). 
% En caso de utilizar un VQ previamente calculado (cero en calc_VQ), habra que pasar los
% parametros del cuantizador: niveles y fronteras.
% En cada uno de los casos posibles (entropia en el dominio espacial, entropia de la seal
% cuantizada en el dominio frecuencial calculando el VQ o con VQ previamente calculado), 
% hay que introducir un cero en los parametros que no sean necesarios. 
%            
%  * Entropia espacial necesita:
%         seal,punto,entorno,Ncuan
%           
%  * Entropia frecuencial con calculo del VQ necesita:
%         seal,punto,entorno,Ncuan,frecuencial?=1,calc_VQ?=1,facfrec,ncontr,nolin?,estab?,
%         B,bitmax,fc,cc,crit,fijarmax?,inclcsf?,frecbaja
%         
%  * Entropia frecuencial con VQ previamente calculado necesita:
%         seal,punto,entorno,Ncuan,frecuencial?=1,calc_VQ?=0,niveles,fronteras,fc,cc,crit,
%         frecbaja
%      
% USO: [H,niveles,fronteras]=entroloc(seal,punto,entorno,Ncuan,frecuencial(1/0)?,calc_VQ(1/0)?,niveles,fronteras,
% facfrec,ncontr,nolin(1/0)?,estab(1/0)?,B,bitmax,fc,cc,crit,fijarmax(0/1)?,inclcsf(0/1)?,frecbaja);

if Ncuan==0
   blo=sacasub(s,pto,lados,0);
   if frei==0
      H=prod(size(blo))*entropia(blo);
      niveles=0;
      fronteras=0; 
   else
      if calvq==1
         % Esto lo comento porque JOTAPIP simplemente no existe (no se que conyo hace esto)
         %[imc,dctc,cod,codint,nceros,ciaf1D,iaf,nuqf,niveles,fronteras,entrop,entropint]=jotapip(blo,facfrec,ncontr,nolin,estab,B,bitmax,fc,cc,crit,fijarmax,inclcsf,frecbaja,1);
         lcuan=size(s,1)/(2^Ncuan);
         [ciaf1D,iaf,nuqf,niveles,fronteras]=vqperc(lcuan,facfrec,ncontr,nolin,estab,B,bitmax,fc,fijarmax,0);
         [imc,dctc,cod,codigo_int,nceros,entrop,entropint]=qdct21(blo,niveles,fronteras,fc,cc,crit,frecbaja,1);         
         clear imc dctc cod codigo_int nceros ciaf1D iaf nuqf 
         H=prod(size(blo))*entropint;
      else
         [imc,dctc,cod,codigo_int,nceros,entrop,entropint]=qdct21(blo,niveles,fronteras,fc,cc,crit,frecbaja,1);
         clear imc dctc cod codigo_int nceros
         H=prod(size(blo))*entropint;
      end
   end   
else
   tam=size(s);
   tam=tam(1);
   lcuan=tam/(2^Ncuan);
   posai=pto-round(lados/2);
   posbd=pto+round(lados/2);
   coorcuanai=floor([(posai(1)-1)/lcuan+1 (posai(2)-1)/lcuan+1]);
   coorcuanbd=floor([(posbd(1)-1)/lcuan+1 (posbd(2)-1)/lcuan+1]);
   if coorcuanai(1)<1
        coorcuanai(1)=1;
   end 
   if coorcuanai(2)<1
        coorcuanai(2)=1;
   end 
   if coorcuanbd(1)>(2^Ncuan)
        coorcuanbd(1)=2^Ncuan;
   end 
   if coorcuanai(2)>(2^Ncuan)
        coorcuanbd(2)=2^Ncuan;
   end 
   H=0;
   for ii=coorcuanai(1):coorcuanbd(1)
       for jj=coorcuanai(2):coorcuanbd(2)
           iii=(ii-1)*lcuan+1+ceil(lcuan/2);
           jjj=(jj-1)*lcuan+1+ceil(lcuan/2); 
           blo=sacasub(s,[iii jjj],[lcuan lcuan],0);
           if frei==0            
              H=H+propor([posai(1) posbd(1);posai(2) posbd(2)],[(ii-1)*lcuan+1 ii*lcuan+1;(jj-1)*lcuan+1 jj*lcuan+1])*prod(lados)*entropia(blo);
           else 
              if calvq==1
                  if (ii==coorcuanai(1))&(jj==coorcuanai(2))
                      % jotapip no existe!!
                      %[imc,dctc,co,codint,nceros,ciaf1D,iaf,nuqf,niveles,fronteras,entrop,entropint]=jotapip(blo,facfrec,ncontr,nolin,estab,B,bitmax,fc,cc,crit,fijarmax,inclcsf,frecbaja,1);
                      lcuan=size(s,1)/(2^Ncuan);
                      [ciaf1D,iaf,nuqf,niveles,fronteras]=vqperc(lcuan,facfrec,ncontr,nolin,estab,B,bitmax,fc,fijarmax,0);
                      [imc,dctc,cod,codigo_int,nceros,entrop,entropint]=qdct21(blo,niveles,fronteras,fc,cc,crit,frecbaja,1);         
                      clear imc dctc cod codigo_int nceros ciaf1D iaf nuqf 
                      hache=entropint;
                      H=H+propor([posai(1) posbd(1);posai(2) posbd(2)],[(ii-1)*lcuan+1 ii*lcuan+1;(jj-1)*lcuan+1 jj*lcuan+1])*prod(lados)*hache;
                  else
                    [imc,dctc,cod,codigo_int,nceros,entrop,entropint]=qdct21(blo,niveles,fronteras,fc,cc,crit,frecbaja,1);
                    clear imc dctc cod codigo_int nceros
                    hache=entropint;
                    H=H+propor([posai(1) posbd(1);posai(2) posbd(2)],[(ii-1)*lcuan+1 ii*lcuan+1;(jj-1)*lcuan+1 jj*lcuan+1])*prod(lados)*hache;
                 end
              else 
                 [imc,dctc,cod,codigo_int,nceros,entrop,entropint]=qdct21(blo,niveles,fronteras,fc,cc,crit,frecbaja,1);
                 clear imc dctc cod codigo_int nceros
                 hache=entropint;
                 H=H+propor([posai(1) posbd(1);posai(2) posbd(2)],[(ii-1)*lcuan+1 ii*lcuan+1;(jj-1)*lcuan+1 jj*lcuan+1])*prod(lados)*hache;
              end
           end
       end
   end
end