function [entrop]=entropi(a)

%'ENTROPIA' calcula la entropia de una matriz aislada refiriendola
% a su propia estadistica, es decir, cuenta todos los valores diferentes 
% (xi) y el numero de veces que aparecen (ni) y calcula cada probabilidad
% individual de cada xi como Pi=ni/N.
% Al final hace el sumatorio correspondiente:
%                             _
%                            | 
%                 Entropia = | P(x) Log2 (P(x)) dx
%                            |
%                           -
%
% USO: [e]=entropi(a);
% 
% NOTA: Ya sabemos que eso no es estrictamente la entropia. Para calcular
% la entropia correctamente habria que conocer la probabilidad de obtener 
% cada valor xi en cada punto i, ademas de la relacion entre el valor de 
% la seal en ese punto con los puntos vecinos.
% Se trata de calcular la entropia de un conjunto de fuentes de informacion
% que no son independientes.
% Evidentemente eso no puede conseguirse a partir de una unica seal, sino
% a partir de un numero suficiente de seales de la misma clase. 


% Convierte a columna  
         a=a(:);                                
% Cuenta el n de elementos
         n=length(a);
% Ordena de menor a mayor
         b=sort(a);
% Pone un pico en los cambios. La altura del pico es proporcional a la magnitud del cambio
         c=filter([1 -1],1,b);                 
% Detecta las posiciones de los los cambios
         d=find(c>0);
         d=[d;n+1];
% Cuenta cual es la distancia entre cambios, es decir, cuanto se repite un dato  
         e=filter([1 -1],1,d);
         e=e(2:length(e));
% La probabilidad es nrepetic/ntotal
         p=e/n;
% Calcula la entropia 
         entrop=-sum(p.*log(p)/log(2));
