function [D,posic,lados,error,H,Hte,Htf,Nestimac,Nentrop]=fluxar13(f2,f1,Ninic,Nmax,Ncuan,fcrit,vec,Npas,fac,frei,calvq,niveles,fronteras,facfrec,ncontr,nolin,estab,B,bitmax,fc,cc,crit,inclcsf,frecbaja);

% FLUXAR13 estima el flujo optico segun una aproximacion jerarquica adaptando localmente la pre-
% cision en el calculo del flujo (adaptando la densidad del campo de desplazamientos) para redu-
% cir una determinada funcion coste. La funcion coste que rige la estimacion esta determinada 
% por el objetivo de la estimacion de movimiento para codificacion de video: el movimiento esti-
% mado debe ser tal que el volumen de la seal transmitida sea minimo satisfaciendose una cierta
% calidad en la reproduccion. 
% De esta forma relacionamos el proceso de estimacion de movimiento con el proceso posterior de
% cuantizacion, modificando la estimacion en funcion de la cuantizacion... (FIN DE LA IDEA!)
%
% FLUXAR13 estima el movimiento de cada subbloque mediante BMA con busqueda en N pasos (BUSCAD1).
% La funcion coste a reducir es el volumen conjunto de la informacion de movimiento (campo de 
% vectores) y la seal de error. La funcion coste sirve para decidir si se incrementa la preci-
% sion de la estimacion del movimiento en una determinada area: si aumentando la precision re-
% ducimos el volumen de la seal a transmitir, se aumenta la precision, sino, se mantiene la es-
% timacion mas grosera.
% A la hora de decidir la division de un bloque particular, se calcula la entropia del campo de
% vectores y de la seal de error en un entorno del bloque considerado con y sin division, sin 
% tener en cuenta interacciones con el resultado de otras posibles divisiones de subbloques (lo
% cual puede ser discutible).
% El calculo de la entropia puede hacerse a pelo en el dominio espacial o en el frecuencial 
% teniendo en cuenta cual va a ser la codificacion posterior: calculando la entropia de la 
% seal cuantizada.
%
% En la codificacion de video se trabaja con dos tipos de estructuras de subbloques para analizar
% los frames: 
%
%    * Una, en el caso de la estimacion del movimiento (que puede ser fija como en el MPEG o 
%      adaptativa como aqui) definida por el vector 'blocs' que contiene los nodos de la des-
%      composicion en arbol.
%
%    * Y otra, en el caso de la cuantizacion que tambien puede ser adaptativa pero que aqui
%      consideraremos fija (como en MPEG). Siendo fija, viene determinada por un solo parame-
%      tro: el nivel de resolucion de la estructura de cuantizacion (Ncuan).
%
% Para calcular la entropia en un entorno del bloque dividido (o por dividir) consideraremos dos
% opciones:
%
%    * Una, calcularla en un entorno estricto del bloque problema (no considerando como va a co-
%      dificarse posteriormente la seal de error).
%
%    * Y otra, calcularla los subbloques de la estructura de cuantificacion que intersecten con 
%      el entorno del bloque problema (considerando 'en parte' como va a codificarse posterior-
%      mente la seal de error).
%
% La seleccion de 'donde' se calcula la entropia se hace atraves del parametro Ncuan:
%
%    * Ncuan > 0  ->  Estamos definiendo una determinada estructura para la cuantizacion.
%                     Dicha estructura sera la utilizada en el calculo de entropias.
%
%    * Ncuan = 0  ->  Estariamos diciendo que se cuantiza todo el frame considerando un solo
%                     bloque. Como esto no suele ser asi (!), utilizamos Ncuan = 0 para selec-
%                     cionar el modo 'calculo de la entropia en un entorno pasando de como se
%                     haga la cuantizacion posterior'. 
%
% El tamao del entorno del bloque considerado se fija mediante el factor 'fcrit' que multipli-
% ca al lado del bloque problema (-> fcrit>1 ).
%
% El calculo de la entropia (funcion coste) en el dominio frecuencial se selecciona introdu-
% ciendo un 1 en la variable 'frecuencial?'.
% El computo de la entropia en el dominio frecuencial lleva asiciado un VQ que habra que
% calcular.
% Para elegir calcular el VQ (porque no se tiene uno previamente calculado) introducir 
% un 1 en la variable 'calc_VQ?'. En este caso habra que pasar los parametros necesarios
% para el diseo del VQ (Ver JOTAPIP). 
% En caso de utilizar un VQ previamente calculado (cero en calc_VQ), habra que pasar los
% parametros del cuantizador: niveles y fronteras.
% En cada uno de los casos posibles (entropia en el dominio espacial, entropia de la seal
% cuantizada en el dominio frecuencial calculando el VQ o con VQ previamente calculado), 
% hay que introducir un cero en los parametros que no sean necesarios. 
%            
%  * Entropia espacial necesita:
%         Ncuan
%           
%  * Entropia frecuencial con calculo del VQ necesita:
%         Ncuan,frecuencial?=1,calc_VQ?=1,facfrec,ncontr,nolin?,estab?,
%         B,bitmax,fc,cc,crit,fijarmax?,inclcsf?,frecbaja
%         
%  * Entropia frecuencial con VQ previamente calculado necesita:
%         Ncuan,frecuencial?=1,calc_VQ?=0,niveles,fronteras,fc,cc,crit,
%         frecbaja
%
% Despues de todo el rollo, el programa devuelve:
%
%    * El campo de vectores desplazamiento.
%    * Las posiciones de dichos vectores.
%    * Las dimensiones de los bloques asociados.  
%    * La seal de error entre los frames.
%    * Las entropias de los bloques en cada nivel de resolucion.
%    * La entropia total en cada nivel de resolucion.
%
% FLUXAR13 da los datos de las entropias de los subbloques del nivel k sin y con division 
% y su posicion en la variable 'H' de tamao *4. En esta variable, los datos de cada nivel
% de resolucion estan separados por una linea de ceros.
%
% USO: [D,pos,lados,error,H,Hte,Htf,Nestimac,Nentrop]=fluxar13(f2,f1,Ninic,Nmax,Ncuan,fcrit,vec,Npas,fac,
%      frecuencial(1/0)?,calc_VQ(1/0)?,niveles,fronteras,facfrec,ncontr,nolin(1/0)?,
%      estab(1/0)?,B,bitmax,fc,cc,crit,inclcsf(0/1)?,frecbaja);
%     

% INICIO DE POSICIONES Y LADOS

tam=size(f1);
media=sum(sum(f1))/prod(tam);
tam=tam(1);

l=4*ones(1,Nmax);
ll=l;
in=zeros(1,Nmax);
for h=2:Nmax
    n=linspace(1,h,h);
    l(h)=sum(2.^(2*n));     % vector con el numero acumulado de subbloques hasta el nivel h 
    ll(h)=l(h)-l(h-1);      % vector con el numero de subbloques en el nivel h 
    in(h)=l(h-1);           % elemento a partir del cual se situa la informacion relativa a los subbloques del nivel h
end

     % Fija el valor inicial del vector de la codificacion en arbol
blocs=zeros(l(Nmax),Nmax);
     % Si hay que empezar desde algun nivel Ninic, se dividen todos los subbloques de niveles previos
if Ninic>1
   blocs(1:l(Ninic-1),Ninic)=ones(l(Ninic-1),1);
end
     % Paso de blocs a posiciones y lados

posick=[];
ladosk=[];

i=Ninic;
for j=1:ll(Ninic)
    if i>1
       e=esta(i,j,blocs(:,i),Nmax,l);
    else
       e=1;
    end
    if e>0
       co=coord(j,[2^i 2^i]); 
       ii=co(1);
       jj=co(2); 
       posick=[posick;[(ii-1)*(tam/(2^i))+1+ceil((tam/(2^i))/2) (jj-1)*(tam/(2^i))+1+ceil((tam/(2^i))/2)]];
       ladosk=[ladosk;[tam/(2^i) tam/(2^i)]];
    end
end
Nestimac=0;
Nentrop=0;
k=Ninic;
H=[];
Hte=[];
Htf=[];

% ESTIMACION UNIFORME

if Ninic==Nmax
   lk=size(posick);
   lk=lk(1);
   for i=1:lk
       Dk(i,:)=buscad1(f2,f1,posick(i,:),vec*ladosk(i,:),Npas,fac);
       Nestimac=Nestimac+1;
   end
   posic=posick;
   lados=ladosk;
   D=Dk;
   error=0;
   H=0;
   Hte=0;
   Htf=0;
   Nestimac=lk;
   Nentrop=0;
else

% BUCLE SOBRE LOS NIVELES DE RESOLUCION

while k<Nmax
      % Prediccion, error y entropia para los bloques de maxima resolucion (los del nivel k)
      % (en la primera pasada tambien se hace estimacion)
         lk=size(posick);
         lk=lk(1);

         if k==Ninic 
            for i=1:lk
                Dk(i,:)=buscad1(f2,f1,posick(i,:),vec*ladosk(i,:),Npas,fac);
                Nestimac=Nestimac+1;
            end
            posic=posick;
            lados=ladosk;
            D=Dk;   
         end

         lt=size(D);
         lt=lt(1);
         estimf2=predigo(f1,D,posic,lados);
         error=f2-estimf2;        
         if frei==1
            error=error+media;
         end
%         tie1=clock;
         for i=1:lk
             if Ncuan>0 
                if k==Ninic
                   if i==1
                      [hache,niveles,fronteras]=entroloc(error,posick(i,:),ceil(fcrit*ladosk(i,:)),Ncuan,frei,calvq,niveles,fronteras,facfrec,ncontr,nolin,estab,B,bitmax,fc,cc,crit,0,inclcsf,frecbaja);
                      Hk(i,1)=hache+prod(size(D))*entropia(D);
                      Nentrop=Nentrop+1;
                   else
                      [hache,niveles,fronteras]=entroloc(error,posick(i,:),ceil(fcrit*ladosk(i,:)),Ncuan,frei,0,niveles,fronteras,facfrec,ncontr,nolin,estab,B,bitmax,fc,cc,crit,0,inclcsf,frecbaja);
                      Hk(i,1)=hache+prod(size(D))*entropia(D);
                      Nentrop=Nentrop+1;
                   end
                else
                   [hache,niveles,fronteras]=entroloc(error,posick(i,:),ceil(fcrit*ladosk(i,:)),Ncuan,frei,0,niveles,fronteras,facfrec,ncontr,nolin,estab,B,bitmax,fc,cc,crit,0,inclcsf,frecbaja);
                   Hk(i,1)=hache+prod(size(D))*entropia(D);
                   Nentrop=Nentrop+1;
                end
             else
                if i==1
                   [hache,niveles,fronteras]=entroloc(error,posick(i,:),ceil(fcrit*ladosk(i,:)),Ncuan,frei,calvq,niveles,fronteras,facfrec,ncontr,nolin,estab,B,bitmax,fc,cc,crit,0,inclcsf,frecbaja);
                   Hk(i,1)=hache+prod(size(D))*entropia(D);
                   Nentrop=Nentrop+1;
                else
                   [hache,niveles,fronteras]=entroloc(error,posick(i,:),ceil(fcrit*ladosk(i,:)),Ncuan,frei,0,niveles,fronteras,facfrec,ncontr,nolin,estab,B,bitmax,fc,cc,crit,0,inclcsf,frecbaja);
                   Hk(i,1)=hache+prod(size(D))*entropia(D);
                   Nentrop=Nentrop+1;
                end
             end
         end
%         tiempo_entropia_nivel_k=etime(clock,tie1)

         [hache,niveles,fronteras]=entroloc(error,[tam/2 tam/2],[tam tam],Ncuan,0,0,niveles,fronteras,facfrec,ncontr,nolin,estab,B,bitmax,fc,cc,crit,0,inclcsf,frecbaja);
         Hte=[Hte;hache+prod(size(D))*entropia(D)];
         if frei==1
             [hache,niveles,fronteras]=entroloc(error,[tam/2 tam/2],[tam tam],Ncuan,1,0,niveles,fronteras,facfrec,ncontr,nolin,estab,B,bitmax,fc,cc,crit,0,inclcsf,frecbaja);
             Htf=[Htf;hache+prod(size(D))*entropia(D)];
         else
             error=error+media; 
             if k==Ninic
                 %'lili'
                 %niveles 
%                 [H,niveles,fronteras]=entroloc(seal,punto,entorno,Ncuan,frecuencial(1/0)?,calc_VQ(1/0)?,niveles,fronteras,facfrec,ncontr,nolin(1/0)?,estab(1/0)?,B,bitmax,fc,cc,crit,fijarmax(0/1)?,inclcsf(0/1)?,frecbaja);
                 [hache,niveles,fronteras]=entroloc(error,[tam/2 tam/2],[tam tam],Ncuan,1,1,niveles,fronteras,facfrec,ncontr,nolin,estab,B,bitmax,fc,cc,crit,0,inclcsf,frecbaja);
             else
                 [hache,niveles,fronteras]=entroloc(error,[tam/2 tam/2],[tam tam],Ncuan,1,0,niveles,fronteras,facfrec,ncontr,nolin,estab,B,bitmax,fc,cc,crit,0,inclcsf,frecbaja);     
             end  
             Htf=[Htf;hache+prod(size(D))*entropia(D)];
             error=error-media; 
         end    
         
     % Inicio de los subbloques que resultaran de las divisiones positivas

         posice=[];
         posicn=[];
         ladosn=[];
         Dn=[];

     % Bucle sobre los bloques del nivel k para ver si se dividen

         comp=linspace(1,lt,lt);
         HHH=[];

         for i=1:lk
%             tie1=clock;
             posicdiv=posic;
             ladosdiv=lados;
             Ddiv=D;
                    
             % Division
                    
             pto1=[posicdiv(lt-lk+i,:)+[-ceil(tam/(2^(k+1)))+ceil(tam/(2^(k+2))) -ceil(tam/(2^(k+1)))+ceil(tam/(2^(k+2)))]];
             pto2=[posicdiv(lt-lk+i,:)+[ceil(tam/(2^(k+1)))-ceil(tam/(2^(k+2))) -ceil(tam/(2^(k+1)))+ceil(tam/(2^(k+2)))]];
             pto3=[posicdiv(lt-lk+i,:)+[-ceil(tam/(2^(k+1)))+ceil(tam/(2^(k+2))) ceil(tam/(2^(k+1)))-ceil(tam/(2^(k+2)))]];
             pto4=[posicdiv(lt-lk+i,:)+[ceil(tam/(2^(k+1)))-ceil(tam/(2^(k+2))) ceil(tam/(2^(k+1)))-ceil(tam/(2^(k+2)))]];
                  
             ptos=[pto1;pto2;pto3;pto4];

             % Estimacion del movimiento de los subbloques del bloque dividido
                 
             for ii=1:4
                 DD(ii,:)=buscad1(f2,f1,ptos(ii,:),vec*[tam/(2^(k+1)) tam/(2^(k+1))],Npas,fac);
                 Nestimac=Nestimac+1;
             end

             cond=(comp~=(lt-lk+i));
             posicdiv=[posicdiv(cond,:);ptos];
             ladosdiv=[ladosdiv(cond,:);(tam/(2^(k+1)))*ones(4,2)];
             Ddiv=[Ddiv(cond,:);DD];
             
             % Prediccion y error con el bloque dividido (dividido de forma aislada)
             
%             tie2=clock;
             estimf2=predigo(f1,Ddiv,posicdiv,ladosdiv);
%             tiempo_prediccion=etime(clock,tie2);
             error=f2-estimf2;             
             if frei==1
                error=error+media;
             end

             % Entropia de la region entorno al bloque dividido en el caso de dividirlo 

%             tie2=clock;  


             [hache,niveles,fronteras]=entroloc(error,posic(lt-lk+i,:),ceil(fcrit*lados(lt-lk+i,:)),Ncuan,frei,0,niveles,fronteras,facfrec,ncontr,nolin,estab,B,bitmax,fc,cc,crit,0,inclcsf,frecbaja);
             HH=hache+prod(size(Ddiv))*entropia(Ddiv);
             HHH=[HHH;HH];
             Nentrop=Nentrop+1;
%             tiempo_entropia_prueba=etime(clock,tie2);
    
             % Merece la pena dividir?
 
             if HH<Hk(i) 
%                1
                posice=[posice;lt-lk+i];
                posicn=[posicn;ptos];
                ladosn=[ladosn;tam/(2^(k+1))*ones(4,2)];
                Dn=[Dn;DD];
             else
                u=0;
             end
%             tiempo_por_bloque=etime(clock,tie1);
%             [tiempo_prediccion tiempo_entropia_prueba tiempo_por_bloque]             
         end
          
     % Actualizacion de los bloques definitivos
         
         % Eliminacion de los datos de los bloques divididos         

             l=size(posice);
             l=l(1);
             ll=size(comp);
             cond=ones(ll(1),ll(2));
             for ii=1:l 
                 cond=cond.*(comp~=posice(ii));
             end
             
             
             posic=posic(logical(cond),:); 
             D=D(logical(cond),:); 
             lados=lados(logical(cond),:);
              
         % Adicion de los datos sobre los subbloques resultantes de las divisiones permitidas
               
             posic=[posic;posicn];
             D=[D;Dn];
             lados=[lados;ladosn];
             
     % Actualizacion de los 'ultimos bloques' (que son los recien divididos)

         if k==Ninic
            H=[Hk HHH posick];
            HHH=[]; 
         else
            H=[H;0 0 0 0;Hk HHH posick];   
            HHH=[];
         end  
         posick=posicn;
         ladosk=ladosn;
         Hk=[];
         k=k+1;

         if k==Nmax
             estimf2=predigo(f1,D,posic,lados);
             error=f2-estimf2;
             if frei==1
                error=error+media;
             end
             [hache,niveles,fronteras]=entroloc(error,[tam/2 tam/2],[tam tam],Ncuan,0,0,niveles,fronteras,facfrec,ncontr,nolin,estab,B,bitmax,fc,cc,crit,0,inclcsf,frecbaja);
             Hte=[Hte;hache+prod(size(D))*entropia(D)];
             if frei==1
                [hache,niveles,fronteras]=entroloc(error,[tam/2 tam/2],[tam tam],Ncuan,1,0,niveles,fronteras,facfrec,ncontr,nolin,estab,B,bitmax,fc,cc,crit,0,inclcsf,frecbaja);
                Htf=[Htf;hache+prod(size(D))*entropia(D)];
             else
                error=error+media; 
                [hache,niveles,fronteras]=entroloc(error,[tam/2 tam/2],[tam tam],Ncuan,1,0,niveles,fronteras,facfrec,ncontr,nolin,estab,B,bitmax,fc,cc,crit,0,inclcsf,frecbaja);
                Htf=[Htf;hache+prod(size(D))*entropia(D)];
                error=error-media; 
             end
         end
end
if frei==1
   error=error-media;
end
end