function mpig223(pel,estim,datmov,Ni,Nm,vec,Np,facD,critdiv,Nquan,facq,opc,dim,traydvq,facfrec,nolin,ncont,estab,B,bitmax,met,norm,fc,cc,crit,frecbaja,fichmov,fichvq,fichcod,fichpel)

% MPIG223 estima el movimiento entre cada par de frames de la secuencia
% de entrada mediante diferentes variantes del BMA (ver CODIAR13) y 
% cuantiza la transformada de coseno espacial de cada frame de la 
% secuencia de correcciones necesaria para compensar los efectos de 
% los errores de prediccion y errores de cuantizacion.
% 
% Se pueden aplicar diferentes diseos de VQ (ver QDCT55).
% Si se elige estimacion de movimiento adaptativa con criterio de
% adaptacion controlado por un cuantizador, ese mismo cuantizador
% es el que se utiliza para cuantizar... podriamos utilizar otro,
% pero eso ya seria morirse introduciendo parametros (p.e. ESTIQUAN).
% Aunque me temo que habra que hacerlo...
%
% MPIG223 devuelve (en los ficheros que se le indiquen):
%
%     - Datos de la estimacion del movimiento:
%            . Flujo optico        (D,posl)
%            . Entropias           (Htte,Httf)
%            . Coste computacional (Ne,Nn)
%  
%     - Datos del cuantizador:
%            . Funcion de asignacion de informacion (iaf) 
%            . Funcion de asignacion integrada      (ciaf1D)
%            . Funciones de cuantizado no uniforme  (nuqf)
%            . Diseo del cuantizador               (niveles,fronteras)
%
%     - Codigos de las transformadas cuantizadas y entropias finales
%       (pelcodq,HHi)
%
%     - Secuencias reconstruida (pelq)
%
%     - Secuencia de fotogramas predichos frame-a-frame partiendo
%       de una determinada correccion previa (que no secuencia sin correcciones!!)
%       (ppp)
%
% NOTAS:
%
%   - Aunque se disponga de un VQ previamente diseado, si hay que
%     introducir fc,cc,crit,f_res_baja
%   - Ojo! Solo peliculas con fotogramas cuadrados con lado potencia de 2
%
% USO: mpig223(pelicula,estim_mov?,[D posilad],Ninic,Nmax,vec_estim,Npas,facD,critdiv(0=esp/1=frec),
%         Nquant,facquan,opc_q(0=previo/1=percep/2=estad),dimens_vq,(VQestad)'traydatosquantiz(confich)',
%         (VQperc)facfrec,(VQperc)nolin,(VQperc)ncontr,(VQperc)estab,B,bitmax,(Vest)metodo_alloc
%         [normDCT param],fc,cc,crit,frecbaja,'fich_result_mov',
%         'fich_result_vq','fich_result_cod','fich_result_pelicula',);
%
% USO: mpig223(pelicula,1,0,4,4,vec_estim,Npas,1,0,4,facquan,1,2,0,1,[1 1 1],100,1,1,10,0,[normDCT param],32,1,3,0.001,'c:\usuarios\mov','c:\usuarios\vq','c:\usuarios\cod','c:\usuarios\pel');

s=size(pel);
nfotp=s(2)/s(1);

for i=1:nfotp-1
    p=sacafot(pel,s(1),s(1),[i i+1]);

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %                            %
    %  ESTIMACION DE MOVIMIENTO  %
    %                            %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    if estim==1
       [D,pospos,laos,e,Hparc,Hte,Htf,Nes,Nen]=fluxar13(p(:,s(1)+1:2*s(1)),p(:,1:s(1)),Ni,Nm,Nquan,facq,vec,Np,facD,critdiv,opc,0,0,facfrec,ncont,nolin,estab,B,bitmax,fc,cc,crit,0,frecbaja);
       if i==1
          sk=p(:,1:s(1));
          DD=D;
          posl=[pospos laos];
          Htte=Hte;
          Httf=Htf;
          Ne=Nes;
          Nn=Nen;    
       else   
          DD=[DD;0 0;D];
          posl=[posl;0 0 0 0;[pospos laos]];
          Htte=[Htte Hte];
          Httf=[Httf Htf];
          Ne=[Ne Nes];
          Nn=[Nn Nen];
       end
    else
       if i==1
           sk=p(:,1:s(1));
       end         
       dat=sacagrup(datmov,i);
       D=dat(:,1:2);
       pospos=dat(:,3:4);   
       laos=dat(:,5:6);
    end

    %%%%%%%%%%%%%%%%
    %              %
    %  PREDICCION  %
    %              %
    %%%%%%%%%%%%%%%%
    
    skm1=predigo(sk,D,pospos,laos);
    if i==1
       ppp=[p(:,1:s(1)) skm1];
    else
       ppp=[ppp skm1];
    end
    ekm1=p(:,s(1)+1:2*s(1))-skm1;
    
    %%%%%%%%%%%%%%%%%%%%%%%
    %                     %
    %  FILTRADO TEMPORAL  %
    %                     %
    %%%%%%%%%%%%%%%%%%%%%%%

    if dim==3
       b=[0.0612 0.1143 0.5428 0.1143 0.0612];
       if i==1
          PPO=p(:,s(1)+1:2*s(1))*b(1)+p(:,1:s(1))*b(2);
          PPP=ppp(:,s(1)+1:2*s(1))*b(1)+ppp(:,1:s(1))*b(2);
       elseif i==2
          PPO=p(:,s(1)+1:2*s(1))*b(1)+p(:,1:s(1))*b(2)+sacafot(pel,s(1),s(1),i-1)*b(3);
          PPP=ppp(:,2*s(1)+1:3*s(1))*b(1)+ppp(:,s(1)+1:2*s(1))*b(2)+ppp(:,1:s(1))*b(3);
       elseif i==3
          PPO=p(:,s(1)+1:2*s(1))*b(1)+p(:,1:s(1))*b(2)+sacafot(pel,s(1),s(1),i-1)*b(3)+sacafot(pel,s(1),s(1),i-2)*b(4);
          PPP=ppp(:,3*s(1)+1:4*s(1))*b(1)+ppp(:,2*s(1)+1:3*s(1))*b(2)+ppp(:,s(1)+1:2*s(1))*b(3)+ppp(:,1:s(1))*b(4);
       else    
          PPO=p(:,s(1)+1:2*s(1))*b(1)+p(:,1:s(1))*b(2)+sacafot(pel,s(1),s(1),i-1)*b(3)+sacafot(pel,s(1),s(1),i-2)*b(4)+sacafot(pel,s(1),s(1),i-3)*b(5);
          PPP=sacafot(ppp,s(1),s(1),i+1)*b(1)+sacafot(ppp,s(1),s(1),i)*b(2)+sacafot(ppp,s(1),s(1),i-1)*b(3)+sacafot(ppp,s(1),s(1),i-2)*b(4)+sacafot(ppp,s(1),s(1),i-3)*b(5);
       end
       skm1=PPP;
       p(:,s(1)+1:2*s(1))=PPO;
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%
    %                         %
    %  CUANTIZACION ESPACIAL  %
    %                         %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%

       if i==1
          [imq,dctq,codq,ciaf1D,iaf,nuqf,niveles,fronteras,H,Hi]=qdct55(p(:,s(1)+1:2*s(1)),Nquan,opc,0,0,facfrec,ncont,nolin,estab,B,bitmax,met,traydvq,fc,cc,crit,0,0,frecbaja,0,norm);
          %size(imq)
          %size(dctq)
          %size(codq)
          %size(ciaf1D)
          %size(iaf)
          %size(nuqf)
          %size(niveles)
          %size(fronteras)
          %size(H)
          %size(Hi)
          clear dctq
          eval(['save ',fichvq,' ciaf1D iaf nuqf niveles fronteras'])
          clear ciaf1D iaf nuqf
          [imqp,dctqp,codqp,ciaf1D,iaf,nuqf,niveles,fronteras,H,Hi]=qdct55(skm1,Nquan,0,niveles,fronteras,facfrec,ncont,nolin,estab,B,bitmax,met,traydvq,fc,cc,crit,0,0,frecbaja,0,norm);
          %size(imqp)
          %size(dctqp)
          %size(codqp)
          %size(ciaf1D)
          %size(iaf)
          %size(nuqf)
          %size(niveles)
          %size(fronteras)
          %size(H)
          %size(Hi)
          clear dctqp
          ekm1=imq-imqp;
          clear imq imqp
          pelq=[p(:,1:s(1)) skm1+ekm1];
          pelcodq=codq-codqp;
          sk=skm1+ekm1;   
          clear codqp codq skm1
          [Hi,ni,fro]=entroloc(pelcodq,floor([s(1) s(1)]/2),[s(1) s(1)],Nquan,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0);
          HHi=Hi;
          HHi/(256*256)
          i
       else
          [imq,dctq,codq,ciaf1D,iaf,nuqf,niveles,fronteras,H,Hi]=qdct55(p(:,s(1)+1:2*s(1)),Nquan,0,niveles,fronteras,facfrec,ncont,nolin,estab,B,bitmax,met,traydvq,fc,cc,crit,0,0,frecbaja,0,norm);
          %size(imq)
          %size(dctq)
          %size(codq)
          %size(ciaf1D)
          %size(iaf)
          %size(nuqf)
          %size(niveles)
          %size(fronteras)
          %size(H)
          %size(Hi)
          clear dctq ciaf1D iaf nuqf
          [imqp,dctqp,codqp,ciaf1D,iaf,nuqf,niveles,fronteras,H,Hi]=qdct55(skm1,Nquan,0,niveles,fronteras,facfrec,ncont,nolin,estab,B,bitmax,met,traydvq,fc,cc,crit,0,0,frecbaja,0,norm);
          %size(imqp)
          %size(dctqp)
          %size(codqp)
          %size(ciaf1D)
          %size(iaf)
          %size(nuqf)
          %size(niveles)
          %size(fronteras)
          %size(H)
          %size(Hi)
          clear dctqp
          ekm1=imq-imqp;
          clear imq imqp
          pelq=[pelq skm1+ekm1];
          codq=codq-codqp;
          sk=skm1+ekm1;
          clear codqp
          clear ekm1
          [Hi,ni,fro]=entroloc(codq,floor([s(1) s(1)]/2),[s(1) s(1)],Nquan,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0);
          HHi=[HHi Hi];
          HHi/(256*256)
          pelcodq=[pelcodq codq];
          clear codq
          i
       end
end

if estim==1
   eval(['save ',fichmov,' DD posl Htte Httf Nn Ne'])
end
eval(['save ',fichcod,' pelcodq HHi'])
clear pelcodq entrocod
eval(['save ',fichpel,' ppp pelq'])