function [nivnu,frontnu]=niv66(numniv,nuqf,contras,uh,csf,fijom);

% NIV66 calcula los niveles de contraste y las fronteras de los 
% intervalos de decision (todo no uniforme) a partir de:
%
%       - El numero de niveles deseado para cada frecuencia dado por
%         round(2^(CIAF*N*N*b))).
%         ATENCION! Aqui se introduce un CIAF1D pero normalizado: 
%         para que al hacer simetria de revolucion y pasar a 2D
%         su volumen sea la unidad (por eso se multiplica por N*N*b)      
%         Asi es como lo da CONST (y como lo impone reducet5).
%
%       - La funcion NUQF para cada frecuencia
%
%       - El dominio de la funcion NUQF, es decir, EL RANGO DE CONTRASTES.
%         ATENCION! el rango de contrastes debe tener igual
%         longitud que NUQF.
%
%       - OPCIONALMENTE el contraste umbral (la CSF) para cada frecuencia
%
% Se supone que existe un elemento de CIAF (y CSF), y una columna de NUQF
% por banda.
% Para una matriz de lado N hay floor(sqrt(2*(N-1)^2)+1) frecuencias dife-
% rentes (bandas).
% 
% USO: [niveles,fronteras]=niv66(num_niveles,nuqf,contrastes,imp_csf?,csf,fijo_max?);
%
% Imponemos nivel1 = C_umbral ?
%  
%          imp_csf? = 1......Si 
%          imp_csf? = 0......No             
%
% Si no se impone la restriccion del contraste umbral, introduzcase
% cualquier cosa en el parametro CSF (no se utiliza)
%
% Fijamos que el ultimo nivel este dado por el maximo de la NUQF?
%
%          fijo_max? = 1......Si
%          fijo_max? = 0......No
%
% Si no fijamos el maximo, Mahalanobis debe mejorar (ver notas)  
 
N=length(numniv);
m=maxi(numniv);

dm=100*maxi(contras)

nivnu=dm*ones(N,m);
frontnu=dm*ones(N,m-1);

if fijom==1
  numniv=numniv+1;
  if uh==0
    for i=1:N
       if numniv(i)==1
          lniv=length(nivnu(i,:));
          nivnu(i,:)=-ones(1,lniv);
       elseif numniv(i)==2
          nivnu(i,1:1)=0;
          frontnu(i,1:1)=dm;
       else               
          a=nuqf(:,i);
          a=a';
          con=contras(:,i);
          con=con';
          nivelesu=linspace(min(a),max(a),numniv(i));
          increm=(nivelesu(2)-nivelesu(1))/2;
          fronteru=linspace(min(a)+increm,max(a)-increm,numniv(i)-1);
          a=monotono(a);
          nivnu(i,1:length(nivelesu))=(interp1(a,con,nivelesu'))';
          frontnu(i,1:length(fronteru))=(interp1(a,con,fronteru'))';
       end 
    end
  else
    for i=1:N
       if numniv(i)==1
          lniv=length(nivnu(i,:));
          nivnu(i,:)=-ones(1,lniv);
       elseif numniv(i)==2
          nivnu(i,1:1)=0;
          frontnu(i,1:1)=dm;
       else               
          a=nuqf(:,i);
          a=a';
          con=contras(:,i);
          con=con';
          b=1./csf(i);
          if b<max(con)
             con=monotono(con);
             mi=interp1(con,a,b');
             nivelesu=linspace(mi,max(a),numniv(i));
             increm=(nivelesu(2)-nivelesu(1))/2;
             fronteru=linspace(min(a)+increm,max(a)-increm,numniv(i)-1);
             a=monotono(a);
             nivnu(i,1:length(nivelesu))=(interp1(a,con,nivelesu'))';
             frontnu(i,1:length(fronteru))=(interp1(a,con,fronteru'))';
          else
             nivnu(i,1:1)=0;
             frontnu(i,1:1)=dm;
          end
      end 
    end
  end
else
  numniv=numniv+1;
  if uh==0
    for i=1:N
       if numniv(i)==1
          lniv=length(nivnu(i,:));
          nivnu(i,:)=-ones(1,lniv);
       elseif numniv(i)==2
          nivnu(i,1:1)=0;
          frontnu(i,1:1)=dm;
       else               
          a=nuqf(:,i);
          a=a';
          D=max(a)-min(a);
          k=numniv(i)-1;
          A1=D*((-3+sqrt(48*k^2-3))/(8*k^2-2));
          increm=(D-A1)/(2*k);
          con=contras(:,i);
          con=con';
          nivelesu=[min(a) linspace(min(a)+A1+increm,max(a)-increm,k-1)];
          fronteru=linspace(min(a)+A1,max(a)-2*increm,k-1);
          a=monotono(a);
          nivnu(i,1:length(nivelesu))=(interp1(a,con,nivelesu'))';
          frontnu(i,1:length(fronteru))=(interp1(a,con,fronteru'))';
       end
    end
  else
    for i=1:N
       if numniv(i)==1
          lniv=length(nivnu(i,:));
          nivnu(i,:)=-ones(1,lniv);
       elseif numniv(i)==2
          nivnu(i,1:1)=0;
          frontnu(i,1:1)=dm;
       else               
          a=nuqf(:,i);
          a=a';
          con=contras(:,i);
          con=con';
          b=1./csf(i);
          if b<max(con)
             con=monotono(con);
             mi=interp1(con,a,b');
             D=max(a)-mi;
             k=numniv(i)-1;
             A1=D*((-3+sqrt(48*k^2-3))/(8*k^2-2));
             increm=(D-A1)/(2*k);
             nivelesu=[mi linspace(mi+A1+increm,max(a)-increm,k-1)];
             fronteru=linspace(mi+A1,max(a)-2*increm,k-1);
             a=monotono(a);
             nivnu(i,1:length(nivelesu))=(interp1(a,con,nivelesu'))';
             frontnu(i,1:length(fronteru))=(interp1(a,con,fronteru'))';
          else
             nivnu(i,1:1)=0;
             frontnu(i,1:1)=dm;
          end
      end
    end
  end
end
%m=size(nivnu);
%m=m(2);
%nivnu=nivnu(:,1:m);
%frontnu=frontnu(:,1:m);