function [imc,dctc,co,cod,ncero,entrop,entrocod]=qdct21(ni,niveles,fronteras,fc,cc,crit,residuo,calentr)

%  QDCT21
%  Compresion de imagenes mediante 'Block Quantization' de los coeficientes
%  de la DCT global.
%  El cuantizador (niveles y fronteras) introducido esta diseado por una 'Funcion de Asignacion
%  de Informacion' (IAF) perceptual o estadistica (segun las constantes elegidas).
% 
%  Despues de la cuantificacion se reordenan los datos mediante 'ZIGZAG' y se 
%  aplica un algoritmo RLE para eliminar los ceros redundantes.
%  Al final se calcula la entropia del codigo resultante.
% 
%  USO: [imc,dctc,cod,codigo_int,nceros,entrop,entropint]
%       =qdct21(nombre im,niveles,fronteras,fc,cc,crit,frec_residuo_baja,calc_entrop?(1/0));
%
%  NOTAS:
%   
%   * Transforma BIEN (por primera vez) los coeficientes de la DCT en con-
%     trastes:
%                                         2*DCT(1,j)
%                    Contraste(1,j) =  ---------------
%                                           DCT(0,0)     
%
%
%                                         2*DCT(i,1)
%                    Contraste(i,1) =  ---------------
%                                           DCT(0,0)     
%
%
%                                         4*DCT(i,j)
%                    Contraste(i,j) =   ---------------  (i>1, j>1)
%                                           DCT(0,0)  
%
%
%   * No cuantifica las componentes cuya frecuencia es inferior a la fijada
%     por el 'residuo de baja'.
%
%   * No cuantifica los coeficientes a los que se les asigne un numero de
%     bits mayor que un cierto limite (bitmax).
%
%   * El fichero de constantes debe contener CON ESOS NOMBRES:
%      
%           ciaf1D, nuqf, contrast (y opcionalmente csf1D y iaf)
%   
%   * Criterios de asignacion: 
%                
%              1...Distancia euclidea 
%                    (minimiza error con UNA imagen)
%
%              2...Al imediatamente inferior
%
%              3...Distancia de Mahalanobis
%                    (minimiza error sobre el conjunto de entrenamiento)     
%

iaf=1;

tf=dct2(ni);
s=size(tf);
s=s(1);

fre=topolar3(fc,s);
cond=fre>residuo;

tfs=tf;
tfs=tfs.*cond;

fla=0;
if tf(1,1)==0
   fla=1;
   tf(1,1)=10;
end

fact2=2/tf(1,1);
fact4=4/tf(1,1);
tfs(1:s,1)=fact2*tfs(1:s,1);
tfs(1,1:s)=fact2*tfs(1,1:s);
tfs(2:s,2:s)=fact4*tfs(2:s,2:s);
%maxi(tfs)

%[tfc,cod,niveles,fronteras]=reducet7(abs(tfs),ciaf1D,B,bitmax,nuqf,contras,incluir,csf1D,crit,fijo);

tfc=zeros(s,s);
cod=zeros(s,s);
for i1=1:s,
    for j1=1:s,
       d=floor(sqrt((i1-1).^2+(j1-1).^2));
       [tfc(i1,j1),cod(i1,j1)]=asignbbb(abs(tfs(i1,j1)),niveles(d+1,:),fronteras(d+1,:),crit);
    end
end
clear niveles
clear fronteras

tfc(1:s,1)=tfc(1:s,1)/fact2;
tfc(1,1:s)=tfc(1,1:s)/fact2;
tfc(2:s,2:s)=tfc(2:s,2:s)/fact4;

c1=(tf>=0);
c2=(tf<0);
tfc=c1.*tfc-c2.*tfc;
cod=c1.*cod-c2.*cod;

anticond=cond<1;
dctc=cc.*anticond.*tf+cond.*tfc;

if fla==1
   dctc(1,1)=0;
   cod(1,1)=0;
else
   dctc(1,1)=cc*tf(1,1);
   cod(1,1)=cc*tf(1,1);
end

imc=idct2(dctc);

co=cod;

if calentr==1
   codigo=zigzag(dctc);
   [codigo,nceros]=rle2(codigo);
   entrop=length(codigo)*entropia(codigo)+length(nceros)*entropia(nceros);
   entrop=entrop/s^2;
   cod=zigzag(cod);
   %enen=entropia(cod);
   [cod,ncero]=rle2(cod);
   entrocod=length(cod)*entropia(cod)+length(ncero)*entropia(ncero);
   entrocod=entrocod/s^2;
else
   entrop=0;
   cod=0;
   ncero=0;
   entrop=0;
   entrocod=0;   
end