function [dctc,co,cod,ncero,entrop,entrocod]=qdct51(tf,niveles,fronteras,fc,cc,crit,residuo,calentr)

%  QDCT51
%  Aplica el diseo del cuantizador previamente definido (con APELES o VQPERC) sobre
%  el bloque que se le pase (ya en el dominio frecuencial y adecuadamente normalizado)
% 
%  Despues de la cuantificacion se reordenan los datos mediante 'ZIGZAG' y se 
%  aplica un algoritmo RLE para eliminar los ceros redundantes.
%  Al final se calcula la entropia del codigo resultante.
%  Saca la DCT quantizada sin desnormalizar.
% 
%  USO: [dctc,cod,codigo_int,nceros,entrop,entropint]
%       =qdct21(nombre im,niveles,fronteras,fc,cc,crit,frec_residuo_baja,calc_entrop?(1/0));
%
%  NOTAS:
%   
%   * No cuantifica las componentes cuya frecuencia es inferior a la fijada
%     por el 'residuo de baja'.
%
%   * No cuantifica los coeficientes a los que se les asigne un numero de
%     bits mayor que un cierto limite (bitmax).
%
%   * Criterios de asignacion: 
%                
%              1...Distancia euclidea 
%                    (minimiza error con UNA imagen)
%
%              2...Al imediatamente inferior
%
%              3...Distancia de Mahalanobis (haciendo caso a las fronteras
%                    que se le pasan -eso es Mahalanobis?-)
%                    (minimiza error sobre el conjunto de entrenamiento)     
%

s=size(tf);
s=s(1);

fre=topolar3(fc,s);
cond=fre>residuo;

tfs=tf;
tfs=tfs.*cond;

%figure(1),plot(tfs)

%[maxi(tfs) mini(tfs)]

tfc=zeros(s,s);
cod=zeros(s,s);
for i1=1:s,
    for j1=1:s,
       d=floor(sqrt((i1-1).^2+(j1-1).^2));
       [tfc(i1,j1),cod(i1,j1)]=asignbbb(abs(tfs(i1,j1)),niveles(d+1,:),fronteras(d+1,:),crit);
%      [niveles(d+1,1) abs(tfs(i1,j1)) tfc(i1,j1)]
%      pause
    end
end
%figure(3),plot(tfc)

clear niveles
clear fronteras

c1=(tf>=0);
c2=(tf<0);
tfc=c1.*tfc-c2.*tfc;
cod=c1.*cod-c2.*cod;

anticond=cond<1;
dctc=cc.*anticond.*tf+cond.*tfc;

co=cod;

if calentr==1
   codigo=zigzag(dctc);
   [codigo,nceros]=rle2(codigo);
   entrop=length(codigo)*entropia(codigo)+length(nceros)*entropia(nceros);
   entrop=entrop/s^2;
   cod=zigzag(cod);
   %enen=entropia(cod);
   [cod,ncero]=rle2(cod);
   entrocod=length(cod)*entropia(cod)+length(ncero)*entropia(ncero);
   entrocod=entrocod/s^2;
else
   entrop=0;
   cod=0;
   ncero=0;
   entrop=0;
   entrocod=0;   
end